/*---------------------------------------------------------------------------*/

/*!  \file  spreads.c
     \brief  soft colloid/dumbbell simulator for polymers.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#include "getopt.h"
#include "condition.h"
#include "iofiles.h"
#include "geometry.h"
#include "blend.h"
#include "luaread.h"
#include "mt.h"
#include "particles.h"
#include "simulator.h"
#include "textread.h"

/*---------------------------------------------------------------------------*/
static char *programstring = "spreads";

static struct option longopts[] =
{
    {"input", required_argument, NULL, 'i'},
    {"position", required_argument, NULL, 'p'},
    {"version", no_argument, NULL, 'v'},
    {"help", no_argument, NULL, 'h'},
    {0, 0, NULL, 0},
};

static char *optstring = "i:p:vh";

static char *helpstring =
"\
Usage: %s [options] [input]\n\
Options:\n\
  -i, --input <file>       use <file> for the input file for the simulator\n\
  -p, --position <file>    read positions of the particles from <file>\n\
  -v, --version            output version information and exit\n\
  -h, --help               display this help and exit\n\
";

static char *copyingstring = COPYRIGHT_STRING;

/*---------------------------------------------------------------------------*/
static int process_option(int argc,char **argv,char **input_lua,
                          char **position_input);

/*---------------------------------------------------------------------------*/
int main(int argc,char **argv)
{
    char *input_lua = DEFAULT_INPUT_FILE;
    char *position_input = NULL;
    simulation_iofiles *iofiles;
    simulation_condition *condition;
    box_geometry *geometry;
    polymer_blend *blend;
    soft_particles *particles;

    if(process_option(argc,argv,&input_lua,&position_input) != 0)
    {
        return EXIT_FAILURE;
    }

    read_input_lua(input_lua,&condition,&iofiles,&geometry,&blend);


    mtinit(condition->seed);

    particles = allocate_soft_particles(blend,geometry);
    if(position_input == NULL)
    {
        set_particle_positions_random(particles,blend,geometry);
    }
    else
    {
        read_positions_text(position_input,particles);
        set_particle_positions_into_box(particles,geometry);
    }

    simulate(condition,iofiles,geometry,blend,particles);

    free_soft_particles(particles);
    free_simulation_condition(condition);
    free_simulation_iofiles(iofiles);
    free_box_geometry(geometry);
    free_polymer_blend(blend);

    return EXIT_SUCCESS;
}

/*---------------------------------------------------------------------------*/
int process_option(int argc,char **argv,char **input_lua,char **position_input)
{
    int c;
    int option_index;
    char *program_name;

    program_name = argv[0];

    do
    {
        option_index = 0;
        c = getopt_long(argc,argv,optstring,longopts,&option_index);

        switch(c)
        {
            case 'i':
                    /*  input file  */
                *input_lua = optarg;
                break;

            case 'p':
                    /*  position input file  */
                *position_input = optarg;
                break;

            case 'v':
                    /*  version  */
                fprintf(stderr,copyingstring,programstring,PACKAGE,VERSION);
                exit(EXIT_SUCCESS);
                break;

            case 'h':
                    /*  help  */
                fprintf(stderr,helpstring,program_name);
                exit(EXIT_SUCCESS);
                break;

            case '?':
                    /*  invalid option  */
                exit(EXIT_FAILURE);
                break;

            case -1:
                break;

            default:
                fprintf(stderr,
                        "error: getopt returned character code 0%o\n", c);
                return 1;
        }
    }
    while(c != -1);


    if(optind < argc)
    {
            /*  input file  */
        *input_lua = argv[optind++];
    }

    return 0;
}

/*---------------------------------------------------------------------------*/
