/*---------------------------------------------------------------------------*/

/*!  \file  dxread.c
     \brief  read field data from DX file generated by spreads.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "xmalloc.h"
#include "dxread.h"

#define STRING_BUFFERSIZE 0x100

/*---------------------------------------------------------------------------*/
typedef enum endian_type
{
    lsb, msb
} endian_type;

typedef struct particles_information
{
    int index;
    endian_type endian;
    int count;
    int offset;
    int size;
    struct particles_information *next;
} particles_information;

/*---------------------------------------------------------------------------*/
static int read_positions_dx_geometry(FILE *fp,int *dimension);
static particles_information *read_positions_dx_particles_information(FILE *fp,int dimension);
static int skip_until_binary_data_block(FILE *fp);
static void free_particles_information_list(particles_information *information);
static void read_positions_dx_particles_data_binary(FILE *fp,void *u,int dimension,
                                                particles_information *information,
                                                long offset_binary);
static void convert_positions_float_to_double(float *u_float,double *u_double,
                                              int count);
static int read_geometry_dx_information(FILE *fp,int *dimension,double *l,
                                        double *delta_l);

/*---------------------------------------------------------------------------*/
int read_positions_dx(char *filename,double ***r,int *dimension,
                      int *n,int **m)
{
    FILE *fp;
    long offset_binary;
    float **r_float;
    particles_information *information,*information_current;
    int i,m_tot;

    if((filename == NULL) || (strlen(filename) == 0))
        return -1;

    fp = fopen(filename,"rb");


    if(read_positions_dx_geometry(fp,dimension) != 0)
    {
        fprintf(stderr,"error: cannot read geometrical information from `%s'.\n",
                filename);

        fclose(fp);
        return -1;
    }


    information = NULL;
    while((information_current = read_positions_dx_particles_information(fp,*dimension))
          != NULL)
    {
        information_current->next = information;
        information = information_current;
    }

    if(information == NULL)
    {
        *n = 0;
        *m = NULL;
        *r = NULL;
        return 0;
    }

    if(skip_until_binary_data_block(fp) != 0)
    {
        fprintf(stderr,"error: cannot find the binary data block in `%s'.\n",
                filename);

        fclose(fp);
        return -1;
    }

    offset_binary = ftell(fp);


    *n = 0;
    m_tot = 0;
    information_current = information;
    while(information_current != NULL)
    {
        (*n)++;
        m_tot += information->count;
        information_current = information_current->next;
    }

    *m = xmalloc(sizeof(int) * (*n));
    information_current = information;
    while(information_current != NULL)
    {
        (*m)[information_current->index] = information_current->count;
        information_current = information_current->next;
    }


    *r =xmalloc(sizeof(double *) * (*n));
    **r =xmalloc(sizeof(double) * m_tot * (*dimension));
    for(i = 1;i < (*n);i++)
    {
        (*r)[i] = (*r)[i - 1] + ((*m)[i - 1]) * (*dimension);
    }
    if(information->size == 4)
    {
        r_float =xmalloc(sizeof(float *) * (*n));
        *r_float =xmalloc(sizeof(float) * m_tot * (*dimension));
        for(i = 1;i < (*n);i++)
        {
            r_float[i] = r_float[i - 1] + ((*m)[i - 1]) * (*dimension);
        }
    }


    information_current = information;
    while(information_current != NULL)
    {
        if(information_current->size == 4)
        {
            read_positions_dx_particles_data_binary(fp,r_float[information_current->index],
                                                    *dimension,information_current,
                                                    offset_binary);
            convert_positions_float_to_double(r_float[information_current->index],
                                              (*r)[information_current->index],
                                              (*m)[information_current->index] * (*dimension));
        }
        else
        {
            read_positions_dx_particles_data_binary(fp,(*r)[information_current->index],
                                                    *dimension,information_current,
                                                    offset_binary);
        }

        information_current = information_current->next;
    }



    fclose(fp);



    if(information->size == 4)
    {
        free(*r_float);
        free(r_float);
    }


    free_particles_information_list(information);

    return 0;
}

/*---------------------------------------------------------------------------*/
int read_positions_dx_geometry(FILE *fp,int *dimension)
{
    static char buf[STRING_BUFFERSIZE];
/*     int n[3]; */
/*     double l[3]; */
    int i,j;
    double d;

        /*  skip object information about "box_data"  */
    fscanf(fp,"object \"box_data\" class gridpositions counts");

        /*  read field geometry information  */
    *dimension = 0;
    while(fscanf(fp,"%s",buf) > 0)
    {
        if(strcmp(buf,"origin") == 0)
        {
            break;
        }
        else if(*dimension == 3)
            return -1;

        (*dimension)++;
    }


    for(i = 0;i < *dimension;i++)
    {
        if(!(fscanf(fp,"%s",buf) > 0))
            return -1;

        sscanf(buf,"%lf",&d);

        if(d != 0.0)
            return -1;
    }

    for(i = 0;i < *dimension;i++)
    {
        if(!(fscanf(fp,"%s",buf) > 0))
            return -1;

        if(strcmp(buf,"delta") != 0)
            return -1;

        for(j = 0;j < *dimension;j++)
        {
            if(!(fscanf(fp,"%s",buf) > 0))
                return -1;

            sscanf(buf,"%lf",&d);
            if((j != i) && (d != 0.0))
            {
                return -1;
            }
        }
    }


        /*  skip object information about "box"  */
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"object \"box\" class field");
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"component \"positions\" value \"box_data\"");

        /*  skip object information about "delta_l"  */
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows");
    if(!(fscanf(fp,"%s",buf) > 0))
        return -1;

    return 0;
}

/*---------------------------------------------------------------------------*/
particles_information *read_positions_dx_particles_information(FILE *fp,int dimension)
{
    static char buf[STRING_BUFFERSIZE];
    int index,count,offset,size;
    particles_information *information;
    endian_type endian;
    int i;

        /*  read object information about "data_particles" */
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"object");

    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(sscanf(buf,"\"data_position%d\"",&index) == 0)
    {
        return NULL;
    }

    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"class array type");


    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(strcmp(buf,"float") == 0)
    {
        size = 4;
    }
    else if(strcmp(buf,"double") == 0)
    {
        size = 8;
    }
    else
    {
        return NULL;
    }

        /*  check data type of positions (vectors)  */
    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(fscanf(fp,"%d",&i) == 0)
        return NULL;
    if(i != 1)
        return NULL;

    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(fscanf(fp,"%d",&i) == 0)
        return NULL;
    if(i != dimension)
        return NULL;


    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;

    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(sscanf(buf,"%d",&count) == 0)
        return NULL;

    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(strcmp(buf,"lsb") == 0)
    {
        endian = lsb;
    }
    else if(strcmp(buf,"msb") == 0)
    {
        endian = msb;
    }
    else
    {
        return NULL;
    }

    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;

    if(!(fscanf(fp,"%s",buf) > 0))
        return NULL;
    if(sscanf(buf,"%d",&offset) == 0)
        return NULL;


        /*  set information for particles  */
    information = xmalloc(sizeof(particles_information));

    information->index = index;
    information->endian = endian;
    information->size = size;
    information->offset = offset;
    information->count = count;
    information->next = NULL;

    return information;
}

/*---------------------------------------------------------------------------*/
int skip_until_binary_data_block(FILE *fp)
{
    static char buf[STRING_BUFFERSIZE];

    while(fscanf(fp,"%s",buf) > 0)
    {
        if(strcmp(buf,"end") == 0)
        {
            fscanf(fp,"\n");
            return 0;
        }
    }

    return -1;
}

/*---------------------------------------------------------------------------*/
void free_particles_information_list(particles_information *information)
{
    if(information->next == NULL)
    {
        free(information);
    }
    else
    {
        free_particles_information_list(information->next);
        free(information);
    }
}

/*---------------------------------------------------------------------------*/
void read_positions_dx_particles_data_binary(FILE *fp,void *u,int dimension,
                                             particles_information *information,
                                             long offset_binary)
{
    fseek(fp,offset_binary + information->offset,SEEK_SET);

    fread(u,information->size,information->count * dimension,fp);
}

/*---------------------------------------------------------------------------*/
void convert_positions_float_to_double(float *u_float,double *u_double,
                                       int count)
{
    int i;

    for(i = 0;i < count;i++)
    {
        u_double[i] = u_float[i];
    }
}

/*---------------------------------------------------------------------------*/
int read_geometry_dx(char *filename,int *dimension,double *l,double *delta_l)
{
    FILE *fp;

    if((filename == NULL) || (strlen(filename) == 0))
        return -1;

    fp = fopen(filename,"rb");


    if(read_geometry_dx_information(fp,dimension,l,delta_l) != 0)
    {
        fprintf(stderr,"error: cannot read geometrical information from `%s'.\n",
                filename);

        fclose(fp);
        return -1;
    }


    fclose(fp);

    return 0;
}

/*---------------------------------------------------------------------------*/
int read_geometry_dx_information(FILE *fp,int *dimension,double *l,
                                 double *delta_l)
{
    static char buf[STRING_BUFFERSIZE];
    int i,j;
    double d;

        /*  skip object information about "box_data"  */
    fscanf(fp,"object \"box_data\" class gridpositions counts");

        /*  read field geometry information  */
    *dimension = 0;
    while(fscanf(fp,"%s",buf) > 0)
    {
        if(strcmp(buf,"origin") == 0)
        {
            break;
        }
        else if(*dimension == 3)
            return -1;

        (*dimension)++;
    }


    for(i = 0;i < *dimension;i++)
    {
        if(!(fscanf(fp,"%s",buf) > 0))
            return -1;

        sscanf(buf,"%lf",&d);

        if(d != 0.0)
            return -1;
    }

    for(i = 0;i < *dimension;i++)
    {
        if(!(fscanf(fp,"%s",buf) > 0))
            return -1;

        if(strcmp(buf,"delta") != 0)
            return -1;

        for(j = 0;j < *dimension;j++)
        {
            if(!(fscanf(fp,"%s",buf) > 0))
                return -1;

            sscanf(buf,"%lf",&d);
            if(j == i)
            {
                l[i] = d;
            }
            else if(d != 0.0)
            {
                return -1;
            }
        }
    }


        /*  skip object information about "box"  */
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"object \"box\" class field");
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"component \"positions\" value \"box_data\"");

        /*  skip object information about "delta_l"  */
    while(fscanf(fp,"\n") > 0)
        ;
    fscanf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows");
    if(!(fscanf(fp,"%s",buf) > 0))
        return -1;
    sscanf(buf,"%lf",&d);
    *delta_l = d;


    return 0;
}

/*---------------------------------------------------------------------------*/
